@@ -109,6 +109,9 @@ typedef enum GeisAttrType
 #define GEIS_GESTURE_TAP     "Tap"
 #define GEIS_GESTURE_TOUCH   "Touch"
 
+/* Extra higher-level gestures. */
+#define GEIS_GESTURE_FLICK   "Flick"
+
 /**
  * @defgroup geis_v1_gesture_types Gesture Types
  * @ingroup geis_v1
@@ -156,6 +159,7 @@ typedef enum GeisAttrType
 
 /* @} */
 
+#define GEIS_GESTURE_ID_FLICK 128
 
 /**
  * @defgroup geis_v1_standar_gesture_types Standard Gesture Types
@@ -209,6 +213,12 @@ typedef enum GeisAttrType
  */
 #define GEIS_GESTURE_TYPE_SYSTEM  "Sysflags"
 
+#define GEIS_GESTURE_TYPE_FLICK1  "Flick,touch=1"
+#define GEIS_GESTURE_TYPE_FLICK2  "Flick,touch=2"
+#define GEIS_GESTURE_TYPE_FLICK3  "Flick,touch=3"
+#define GEIS_GESTURE_TYPE_FLICK4  "Flick,touch=4"
+#define GEIS_GESTURE_TYPE_FLICK5  "Flick,touch=5"
+
 /* @} */
 
 /* Standard fundamental gesture attributes */
@@ -220,6 +230,8 @@ typedef enum GeisAttrType
 #define GEIS_GESTURE_ATTRIBUTE_BOUNDINGBOX_X2   "boundingbox x2"
 #define GEIS_GESTURE_ATTRIBUTE_BOUNDINGBOX_Y2   "boundingbox y2"
 #define GEIS_GESTURE_ATTRIBUTE_CHILD_WINDOW_ID  "child window id"
+#define GEIS_GESTURE_ATTRIBUTE_CENTROID_X       "centroid x"
+#define GEIS_GESTURE_ATTRIBUTE_CENTROID_Y       "centroid y"
 #define GEIS_GESTURE_ATTRIBUTE_DELTA_X          "delta x"
 #define GEIS_GESTURE_ATTRIBUTE_DELTA_Y          "delta y"
 #define GEIS_GESTURE_ATTRIBUTE_DEVICE_ID        "device id"
@@ -1046,16 +1058,22 @@ GEIS_API GeisStatus geis_next_event(Geis geis, GeisEvent *event);
  * @def GEIS_DEVICE_ATTRIBUTE_NAME
  * The name of the input device.  Not guaranteed unique.
  *
+ * The attribute value is of type GeisString.
+ *
  * @def GEIS_DEVICE_ATTRIBUTE_ID
  * The unique integer ID of the device.  Guaranteed unique within a Geis
  * instance.
  *
+ * The attribute values is of type GeisInteger.
+ *
  * @def GEIS_DEVICE_ATTRIBUTE_TOUCHES
  * The maximum number of touches a device is capable of reporting.
  * This integer is the number if simultaneous touches the device claims to be
  * able to detect if it is a multi-touch device.  A value of zero indicates the
  * maximum number of touches can not be determined.
  *
+ * The attribute value is of type GeisInteger.
+ *
  * @def GEIS_DEVICE_ATTRIBUTE_DIRECT_TOUCH
  * Indicates the device is a direct touch device.
  * The present of this boolean attribute with a value of GEIS_TRUE indicates the
@@ -1063,17 +1081,63 @@ GEIS_API GeisStatus geis_next_event(Geis geis, GeisEvent *event);
  * otherwise it is an indirect touch device (such as a touchpad) or not a touch
  * device at all.
  *
+ * The attribute value is of type GeisBoolean.
+ *
  * @def GEIS_DEVICE_ATTRIBUTE_INDEPENDENT_TOUCH
  * Indicates the device is an independent touch device.
  * The presence of this boolean attribute with a value of GEIS_TRUE indicates
  * the device is an independent touch device (for example, an Apple MagicMouse).
  * Other multi-touch devices should report GEIS_FALSE.
+ *
+ * The attribute value is of type GeisBoolean.
+ *
+ * @def GEIS_DEVICE_ATTRIBUTE_MIN_X
+ * The lower bound of the X-axis (nominally horizontal) coordinate values
+ * reported by the device.
+ *
+ * The attribute values is of type GeisFloat.
+ *
+ * @def GEIS_DEVICE_ATTRIBUTE_MAX_X
+ * The upper bound of the X-axis (nominally horizontal) coordinate values
+ * reported by the device.
+ *
+ * The attribute values is of type GeisFloat.
+ *
+ * @def GEIS_DEVICE_ATTRIBUTE_RES_X
+ * The resolution of the X-axis (nominally horizontal) coordinate values
+ * reported by the device.
+ *
+ * The attribute values is of type GeisFloat.
+ *
+ * @def GEIS_DEVICE_ATTRIBUTE_MIN_Y
+ * The lower bound of the Y-axis (nominally vertical) coordinate values
+ * reported by the device.
+ *
+ * The attribute values is of type GeisFloat.
+ *
+ * @def GEIS_DEVICE_ATTRIBUTE_MAX_Y
+ * The upper bound of the Y-axis (nominally vertical) coordinate values
+ * reported by the device.
+ *
+ * The attribute values is of type GeisFloat.
+ *
+ * @def GEIS_DEVICE_ATTRIBUTE_RES_Y
+ * The resolution of the Y-axis (nominally vertical) coordinate values
+ * reported by the device.
+ *
+ * The attribute values is of type GeisFloat.
  */
 #define GEIS_DEVICE_ATTRIBUTE_NAME              "device name"
 #define GEIS_DEVICE_ATTRIBUTE_ID                "device id"
 #define GEIS_DEVICE_ATTRIBUTE_TOUCHES           "device touches"
 #define GEIS_DEVICE_ATTRIBUTE_DIRECT_TOUCH      "direct touch"
 #define GEIS_DEVICE_ATTRIBUTE_INDEPENDENT_TOUCH "independent touch"
+#define GEIS_DEVICE_ATTRIBUTE_MIN_X             "device X minimum"
+#define GEIS_DEVICE_ATTRIBUTE_MAX_X             "device X maximum"
+#define GEIS_DEVICE_ATTRIBUTE_RES_X             "device X resolution"
+#define GEIS_DEVICE_ATTRIBUTE_MIN_Y             "device Y minimum"
+#define GEIS_DEVICE_ATTRIBUTE_MAX_Y             "device Y maximum"
+#define GEIS_DEVICE_ATTRIBUTE_RES_Y             "device Y resolution"
 
 /* @} */
 
@@ -1399,7 +1463,8 @@ typedef enum _GeisFilterFacility
 {
   GEIS_FILTER_DEVICE   = 1000,  /**< Filters on device attributes. */
   GEIS_FILTER_CLASS    = 2000,  /**< Filters on gesture attributes. */
-  GEIS_FILTER_REGION   = 3000   /**< Filters on region attributes. */
+  GEIS_FILTER_REGION   = 3000,  /**< Filters on region attributes. */
+  GEIS_FILTER_SPECIAL  = 5000   /**< Filters on special attributes. */
 } GeisFilterFacility;
 
 /**
