@@ -2,6 +2,13 @@
  * @file libutouch-geis/geis.c
  * @brief implementation of the uTouch GEIS v2.0 API instance
  *
+ * The GEIS API object encapsulates an entire persistent session with the
+ * gesture recognition engine.  An application should be able to support
+ * multiple simultaneous instances of the GEIS API abject (although it would be
+ * foolish to design an application that way).
+ */
+
+/*
  * Copyright 2010, 2011 Canonical Ltd.
  *
  * This library is free software; you can redistribute it and/or modify it under
@@ -14,9 +21,8 @@
  * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
  * details.
  *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #include "geis_config.h"
 #include "geis_private.h"
@@ -31,14 +37,40 @@
 #include "geis_error.h"
 #include "geis_event.h"
 #include "geis_event_queue.h"
+#include "geis_gesture_flick.h"
 #include "geis_logging.h"
 #include <stdarg.h>
 #include <string.h>
-#include "test_fixture/geis_backend_test_fixture.h"
 #include <unistd.h>
-#include "xcb/geis_xcb_backend.h"
 
 
+/*
+ * An internal struct to collect filterable attributes for facilities.
+ */
+typedef struct FilterableAttributeBag
+{
+  GeisFilterableAttribute store;
+  GeisSize                size;
+  GeisSize                count;
+} *FilterableAttributeBag;
+
+/*
+ * An internal structure to track processing callbacks in some order.
+ */
+typedef struct GeisProcessingEntry *GeisProcessingEntry;
+
+struct GeisProcessingEntry 
+{
+  GeisProcessingEntry     next;
+  int                     priority;
+  GeisProcessingCallback  event_callback;
+  void                   *context;
+};
+
+
+/*
+ * The API instance.
+ */
 struct _Geis
 {
   GeisRefCount            refcount;
@@ -48,15 +80,21 @@ struct _Geis
   GeisBackend             backend;
   GeisEventQueue          input_event_queue;
   int                     input_event_signal_pipe[2];
+  GeisProcessingEntry     processing_callbacks;
   GeisEventQueue          output_event_queue;
   GeisEventCallback       output_event_callback;
   void                   *output_event_callback_context;
-  GeisDeviceBag           devices;
-  GeisEventCallback       device_event_callback;
-  void                   *device_event_callback_context;
-  GeisGestureClassBag     gesture_classes;
   GeisEventCallback       class_event_callback;
   void                   *class_event_callback_context;
+  FilterableAttributeBag  class_filterable_attributes;
+  GeisGestureClassBag     gesture_classes;
+  GeisEventCallback       device_event_callback;
+  void                   *device_event_callback_context;
+  FilterableAttributeBag  device_filterable_attributes;
+  GeisDeviceBag           devices;
+  FilterableAttributeBag  region_filterable_attributes;
+  FilterableAttributeBag  special_filterable_attributes;
+  GeisGestureFlick        flick;
 };
 
 
@@ -66,7 +104,7 @@ struct _Geis
 static void
 _default_output_event_callback(Geis       geis,
                                GeisEvent  event,
-                               void      *context __attribute__((unused)))
+                               void      *context GEIS_UNUSED)
 {
   geis_debug("posting output event");
   geis_event_queue_enqueue(geis->output_event_queue, event);
@@ -80,7 +118,9 @@ static GeisBoolean
 _device_event_handler(Geis geis, GeisEvent event)
 {
   GeisBoolean handled = GEIS_FALSE;
-  GeisAttr attr = geis_event_attr_by_name(event, GEIS_EVENT_ATTRIBUTE_DEVICE);
+  GeisAttr attr = NULL;
+
+  attr = geis_event_attr_by_name(event, GEIS_EVENT_ATTRIBUTE_DEVICE);
   if (!attr)
   {
     geis_warning("invalid device event received from back end.");
@@ -118,8 +158,9 @@ static GeisBoolean
 _class_event_handler(Geis geis, GeisEvent event)
 {
   GeisBoolean handled = GEIS_FALSE;
+  GeisAttr attr = NULL;
 
-  GeisAttr attr = geis_event_attr_by_name(event, GEIS_EVENT_ATTRIBUTE_CLASS);
+  attr = geis_event_attr_by_name(event, GEIS_EVENT_ATTRIBUTE_CLASS);
   if (!attr)
   {
     geis_warning("invalid class event received from back end.");
@@ -156,13 +197,12 @@ final_exit:
 
 /*
  * Filters and transforms raw gesture events into cooked gesture events.
- *
- * For now, does nothing except copy from the input queueu to the output queue.
  */
 static void
 _input_event_handler(int fd, GeisBackendMultiplexorEvent mux_ev, void *context)
 {
   Geis geis = (Geis)context;
+  GeisProcessingEntry cb;
 
   if (mux_ev == GEIS_BE_MX_READ_AVAILABLE)
   {
@@ -197,6 +237,26 @@ _input_event_handler(int fd, GeisBackendMultiplexorEvent mux_ev, void *context)
 	  break;
       }
 
+      for (cb = geis->processing_callbacks; cb; cb = cb->next)
+      {
+	switch (cb->event_callback(event, cb->context))
+	{
+	  case GEIS_PROCESSING_DISPOSE_EVENT:
+	    geis_event_delete(event);
+	    goto final_exit;
+	    break;
+	  case GEIS_PROCESSING_COMPLETE:
+	    goto processing_complete;
+	    break;
+	  case GEIS_PROCESSING_FAIL:
+	    geis_warning("processig error in event handler");
+	    break;
+	  default:
+	    break;
+	}
+      }
+
+processing_complete:
       if (!handled)
       {
 	geis->output_event_callback(geis,
@@ -205,8 +265,161 @@ _input_event_handler(int fd, GeisBackendMultiplexorEvent mux_ev, void *context)
       }
     }
   }
+
+final_exit:
+  return;
 }
 
+
+/*
+ * Constructs a new filterable attribute bag.
+ */
+static FilterableAttributeBag
+_filterable_attribute_bag_new()
+{
+  FilterableAttributeBag bag = calloc(1, sizeof(struct FilterableAttributeBag));
+  if (!bag)
+  {
+    geis_error("failed to allocate filterable attribute bag");
+    goto final_exit;
+  }
+
+  bag->size = 2;
+  bag->count = 0;
+
+  bag->store = calloc(1, sizeof(struct GeisFilterableAttribute));
+  if (!bag)
+  {
+    geis_error("failed to allocate filterable attribute bag store");
+    goto unwind_bag;
+  }
+  goto final_exit;
+
+unwind_bag:
+  free(bag);
+  bag = NULL;
+final_exit:
+  return bag;
+}
+
+
+/*
+ * Destroys a filterable attribute bag.
+ */
+static void
+_filterable_attribute_bag_delete(FilterableAttributeBag bag)
+{
+  if (bag)
+  {
+    GeisSize i;
+    for (i = 0; i < bag->count; ++i)
+    {
+      free((char *)bag->store[i].name);
+    }
+    free(bag->store);
+  }
+  free(bag);
+}
+
+
+static void
+_filterable_attribute_copy(GeisFilterableAttribute src,
+                           GeisFilterableAttribute dst)
+{
+  dst->name = strdup(src->name);
+  dst->type = src->type;
+  dst->add_term_callback = src->add_term_callback;
+  dst->add_term_context  = src->add_term_context;
+}
+
+
+static void
+_filterable_attribute_bag_insert(FilterableAttributeBag  bag,
+                                 GeisFilterableAttribute fa)
+{
+  GeisSize new_count = bag->count + 1;
+  if (new_count >= bag->size)
+  {
+    GeisSize new_size = bag->size * 2;
+    GeisSize allocation_size = new_size * sizeof(struct GeisFilterableAttribute);
+    GeisFilterableAttribute new_store = realloc(bag->store, allocation_size);
+    if (!new_store)
+    {
+      geis_error("failed to reallocate filterable attribute bag store");
+    }
+    else
+    {
+      bag->store = new_store;
+      bag->size = new_size;
+    }
+  }
+  _filterable_attribute_copy(fa, &bag->store[bag->count]);
+  bag->count = new_count;
+}
+
+
+/*
+ * Applies the back end callback to the back end token for each filterable
+ * attribute with a name matching the argument.
+ */
+static GeisStatus
+_filterable_attribute_bag_foreach(FilterableAttributeBag  bag,
+                                  GeisBackendToken        token,
+                                  GeisString              name,
+                                  GeisFilterOperation     op,
+                                  void                   *value)
+{
+  GeisStatus status = GEIS_STATUS_SUCCESS;
+  GeisSize i;
+  for (i = 0; i < bag->count; ++i)
+  {
+    GeisFilterableAttribute fa = &bag->store[i];
+    if (0 == strcmp(fa->name, name))
+    {
+      status = fa->add_term_callback(token, fa->add_term_context, name, op, value);
+    }
+  }
+  return status;
+}
+
+
+/*
+ * Runs filters for each regostered token in a facility.
+ */
+GeisStatus
+geis_filterable_attribute_foreach(Geis geis,
+                                  GeisFilterFacility  facility,
+                                  GeisBackendToken    token,
+                                  GeisString          name,
+                                  GeisFilterOperation op,
+                                  void               *value)
+{
+  GeisStatus status = GEIS_STATUS_UNKNOWN_ERROR;
+  switch (facility)
+  {
+    case GEIS_FILTER_CLASS:
+      status = _filterable_attribute_bag_foreach(geis->class_filterable_attributes,
+                                                 token, name, op, value);
+      break;
+    case GEIS_FILTER_DEVICE:
+      status = _filterable_attribute_bag_foreach(geis->device_filterable_attributes,
+                                                 token, name, op, value);
+      break;
+    case GEIS_FILTER_REGION:
+      status = _filterable_attribute_bag_foreach(geis->region_filterable_attributes,
+                                                 token, name, op, value);
+      break;
+    case GEIS_FILTER_SPECIAL:
+      status = _filterable_attribute_bag_foreach(geis->special_filterable_attributes,
+                                                 token, name, op, value);
+      break;
+    default:
+      break;
+  }
+  return status;
+}
+
+
 /**
  * Creates a new empty Geis API instance.
  */
@@ -268,28 +481,64 @@ geis_new_empty()
   }
   geis->output_event_callback = _default_output_event_callback;
 
-  geis->devices = geis_device_bag_new();
-  if (!geis->devices)
+  geis->class_filterable_attributes = _filterable_attribute_bag_new();
+  if (!geis->class_filterable_attributes)
   {
     geis_error_push(NULL, GEIS_STATUS_UNKNOWN_ERROR);
-    geis_error("creation of geis device bag failed");
+    geis_error("creation of geis gesture class bag failed");
     goto unwind_output_queue;
   }
-  geis->device_event_callback = _default_output_event_callback;
 
   geis->gesture_classes = geis_gesture_class_bag_new();
   if (!geis->gesture_classes)
   {
     geis_error_push(NULL, GEIS_STATUS_UNKNOWN_ERROR);
     geis_error("creation of geis gesture class bag failed");
-    goto unwind_device_bag;
+    goto unwind_class_attrs;
   }
   geis->class_event_callback = _default_output_event_callback;
 
+  geis->device_filterable_attributes = _filterable_attribute_bag_new();
+  if (!geis->device_filterable_attributes)
+  {
+    geis_error_push(NULL, GEIS_STATUS_UNKNOWN_ERROR);
+    geis_error("creation of geis device bag failed");
+    goto unwind_class_bag;
+  }
+
+  geis->devices = geis_device_bag_new();
+  if (!geis->devices)
+  {
+    geis_error_push(NULL, GEIS_STATUS_UNKNOWN_ERROR);
+    geis_error("creation of geis device bag failed");
+    goto unwind_device_attrs;
+  }
+  geis->device_event_callback = _default_output_event_callback;
+
+  geis->region_filterable_attributes = _filterable_attribute_bag_new();
+  if (!geis->region_filterable_attributes)
+  {
+    goto unwind_device_bag;
+  }
+
+  geis->special_filterable_attributes = _filterable_attribute_bag_new();
+  if (!geis->special_filterable_attributes)
+  {
+    goto unwind_region_attrs;
+  }
+
   goto final_exit;
 
+unwind_region_attrs:
+  _filterable_attribute_bag_delete(geis->region_filterable_attributes);
 unwind_device_bag:
   geis_device_bag_delete(geis->devices);
+unwind_device_attrs:
+  _filterable_attribute_bag_delete(geis->device_filterable_attributes);
+unwind_class_bag:
+  geis_gesture_class_bag_delete(geis->gesture_classes);
+unwind_class_attrs:
+  _filterable_attribute_bag_delete(geis->class_filterable_attributes);
 unwind_output_queue:
   geis_event_queue_delete(geis->output_event_queue);
 unwind_input_signal_pipe:
@@ -362,20 +611,18 @@ _set_valist(Geis geis, GeisString init_arg_name, va_list varargs)
     init_arg_name = va_arg(varargs, GeisString);
   }
 
-  if (back_end_type ==BACK_END_TYPE_MOCK_ENGINE)
+  if (back_end_type == BACK_END_TYPE_MOCK_ENGINE)
   {
-    geis->backend = geis_backend_new_test_fixture(geis,
-                                                  track_devices,
-                                                  track_classes);
+    geis->backend = geis_backend_by_name(geis, GEIS_INIT_UTOUCH_MOCK_ENGINE);
   }
   else if (back_end_type ==BACK_END_TYPE_XCB)
   {
-    geis->backend = geis_xcb_backend_new(geis, track_devices, track_classes);
+    geis->backend = geis_backend_by_name(geis, GEIS_INIT_UTOUCH_XCB);
   }
   else
   {
     geis_warning("back end not specified, defaulting to XCB");
-    geis->backend = geis_xcb_backend_new(geis, track_devices, track_classes);
+    geis->backend = geis_backend_by_name(geis, GEIS_INIT_UTOUCH_XCB);
   }
   if (!geis->backend)
   {
@@ -412,14 +659,25 @@ geis_new(GeisString init_arg_name, ...)
     geis_error("can not initialize GEIS API");
     geis_delete(geis);
     geis = NULL;
+    goto final_exit;
   }
+  geis->flick = geis_gesture_flick_new(geis);
+
+final_exit:
   return geis;
 }
 
 
 static void _geis_destroy(Geis geis)
 {
+  geis_debug("begins");
+  geis_gesture_flick_delete(geis->flick);
+  _filterable_attribute_bag_delete(geis->special_filterable_attributes);
+  _filterable_attribute_bag_delete(geis->region_filterable_attributes);
   geis_device_bag_delete(geis->devices);
+  _filterable_attribute_bag_delete(geis->device_filterable_attributes);
+  geis_gesture_class_bag_delete(geis->gesture_classes);
+  _filterable_attribute_bag_delete(geis->class_filterable_attributes);
   geis_event_queue_delete(geis->output_event_queue);
   close(geis->input_event_signal_pipe[0]);
   close(geis->input_event_signal_pipe[1]);
@@ -545,6 +803,52 @@ geis_register_class_callback(Geis               geis,
 }
 
 
+/*
+ * Registers an event-processing callback.
+ */
+void
+geis_register_processing_callback(Geis                    geis,
+                                  int                     priority,
+                                  GeisProcessingCallback  event_callback,
+                                  void                   *context)
+{
+  GeisProcessingEntry new_entry = calloc(1, sizeof(struct GeisProcessingEntry));
+  new_entry->priority = priority;
+  new_entry->event_callback = event_callback;
+  new_entry->context = context;
+
+  if (!geis->processing_callbacks)
+  {
+    geis->processing_callbacks = new_entry;
+  }
+  else
+  {
+    GeisProcessingEntry cb, prev;
+    for (cb = geis->processing_callbacks; cb; cb = cb->next)
+    {
+      if (priority < cb->priority)
+      {
+	if (cb == geis->processing_callbacks)
+	{
+	  geis->processing_callbacks = new_entry;
+	}
+	else
+	{
+	  new_entry->next = prev->next;
+	  prev->next = new_entry;
+	}
+	break;
+      }
+      prev = cb;
+    }
+    if (!cb)
+    {
+      prev->next = new_entry;
+    }
+  }
+}
+
+
 /*
  * Registers an application-supplied event callback.
  */
@@ -651,40 +955,150 @@ geis_subscription_bag(Geis geis)
 
 
 /*
- * Activates a subscription.
+ * Creates a new backend token.
  */
-GeisStatus
-geis_activate_subscription(Geis geis, GeisSubscription sub)
+GeisBackendToken
+geis_backend_token_new(Geis geis, GeisBackendTokenInitState init_state)
 {
-  return geis_backend_subscribe(geis->backend, sub);
+  return geis_backend_create_token(geis->backend, init_state);
 }
 
 
 /*
- * Deactivates a subscription.
+ * Resgisters a new gesture class with the API.
  */
-GeisStatus
-geis_deactivate_subscription(Geis geis, GeisSubscription sub)
+void
+geis_register_gesture_class(Geis                    geis,
+                            GeisGestureClass        gesture_class,
+                            GeisSize                filterable_attr_count,
+                            GeisFilterableAttribute filterable_attrs)
 {
-  return geis_backend_unsubscribe(geis->backend, sub);
+  GeisAttr attr = NULL;
+  GeisEvent event = NULL;
+  GeisSize i;
+
+  /* Add the filterable attributes. */
+  for (i = 0; i < filterable_attr_count; ++i)
+  {
+    _filterable_attribute_bag_insert(geis->class_filterable_attributes,
+                                     &filterable_attrs[i]);
+  }
+
+  geis_debug("registering class \"%s\" id %d",
+             geis_gesture_class_name(gesture_class),
+             geis_gesture_class_id(gesture_class));
+
+  /* Add a "new gesture class" event to the processing queue. */
+  event = geis_event_new(GEIS_EVENT_CLASS_AVAILABLE);
+  attr = geis_attr_new(GEIS_EVENT_ATTRIBUTE_CLASS,
+                       GEIS_ATTR_TYPE_POINTER,
+                       gesture_class);
+  geis_event_add_attr(event, attr);
+  geis_post_event(geis, event);
+}
+
+
+GeisGestureClassBag
+geis_gesture_classes(Geis geis)
+{
+  return geis->gesture_classes;
+}
+
+
+/*
+ * Resgisters a new input devicve with the API.
+ */
+void
+geis_register_device(Geis                    geis,
+                     GeisDevice              device,
+                     GeisSize                filterable_attr_count,
+                     GeisFilterableAttribute filterable_attrs)
+{
+  GeisAttr attr = NULL;
+  GeisEvent event = NULL;
+  GeisSize i;
+
+  geis_debug("registering device \"%s\" id %d",
+             geis_device_name(device),
+             geis_device_id(device));
+
+  /* Add the filterable attributes. */
+  for (i = 0; i < filterable_attr_count; ++i)
+  {
+    _filterable_attribute_bag_insert(geis->device_filterable_attributes,
+                                     &filterable_attrs[i]);
+  }
+
+  /* Add a "new device" event to the processing queue. */
+  event = geis_event_new(GEIS_EVENT_DEVICE_AVAILABLE);
+  attr = geis_attr_new(GEIS_EVENT_ATTRIBUTE_DEVICE,
+                       GEIS_ATTR_TYPE_POINTER,
+                       device);
+  geis_event_add_attr(event, attr);
+  geis_post_event(geis, event);
+}
+
+
+GeisDeviceBag
+geis_devices(Geis geis)
+{
+  return geis->devices;
+}
+
+
+/*
+ * Resgisters a new region with the API.
+ */
+void
+geis_register_region(Geis                    geis,
+                     GeisRegion              region GEIS_UNUSED,
+                     GeisSize                filterable_attr_count,
+                     GeisFilterableAttribute filterable_attrs)
+{
+  GeisSize i;
+
+  geis_debug("registering region");
+
+  /* Add the filterable attributes. */
+  for (i = 0; i < filterable_attr_count; ++i)
+  {
+    _filterable_attribute_bag_insert(geis->region_filterable_attributes,
+                                     &filterable_attrs[i]);
+  }
+}
+
+
+/*
+ * Resgisters a new special filter with the API.
+ */
+void
+geis_register_special(Geis                    geis,
+                      GeisSize                filterable_attr_count,
+                      GeisFilterableAttribute filterable_attrs)
+{
+  GeisSize i;
+
+  geis_debug("registering feature");
+
+  /* Add the filterable attributes. */
+  for (i = 0; i < filterable_attr_count; ++i)
+  {
+    _filterable_attribute_bag_insert(geis->special_filterable_attributes,
+                                     &filterable_attrs[i]);
+  }
 }
 
 
 GeisAttrType
 geis_get_device_attr_type(Geis geis, GeisString attr_name)
 {
-  GeisSize i = 0;
-  for (; i < geis_device_bag_count(geis->devices); ++i)
+  GeisSize i;
+  for (i = 0; i < geis->device_filterable_attributes->count; ++i)
   {
-    GeisDevice device = geis_device_bag_device(geis->devices, i);
-    GeisSize j = 0;
-    for (; j < geis_device_attr_count(device); ++j)
+    GeisFilterableAttribute fa = &geis->device_filterable_attributes->store[i];
+    if (0 == strcmp(attr_name, fa->name))
     {
-      GeisAttr attr = geis_device_attr(device, j);
-      if (0 == strcmp(geis_attr_name(attr), attr_name))
-      {
-	return geis_attr_type(attr);
-      }
+      return fa->type;
     }
   }
   return GEIS_ATTR_TYPE_UNKNOWN;
@@ -694,18 +1108,13 @@ geis_get_device_attr_type(Geis geis, GeisString attr_name)
 GeisAttrType
 geis_get_class_attr_type(Geis geis, GeisString attr_name)
 {
-  GeisSize i = 0;
-  for (; i < geis_gesture_class_bag_count(geis->gesture_classes); ++i)
+  GeisSize i;
+  for (i = 0; i < geis->class_filterable_attributes->count; ++i)
   {
-    GeisGestureClass gesture_class = geis_gesture_class_bag_gesture_class(geis->gesture_classes, i);
-    GeisSize j = 0;
-    for (; j < geis_gesture_class_attr_count(gesture_class); ++j)
+    GeisFilterableAttribute fa = &geis->class_filterable_attributes->store[i];
+    if (0 == strcmp(attr_name, fa->name))
     {
-      GeisAttr attr = geis_gesture_class_attr(gesture_class, j);
-      if (0 == strcmp(geis_attr_name(attr), attr_name))
-      {
-	return geis_attr_type(attr);
-      }
+      return fa->type;
     }
   }
   return GEIS_ATTR_TYPE_UNKNOWN;
@@ -715,9 +1124,31 @@ geis_get_class_attr_type(Geis geis, GeisString attr_name)
 GeisAttrType
 geis_get_region_attr_type(Geis geis, GeisString attr_name)
 {
-  if (0 == strcmp(attr_name, GEIS_REGION_ATTRIBUTE_WINDOWID))
+  GeisSize i;
+  for (i = 0; i < geis->region_filterable_attributes->count; ++i)
   {
-    return GEIS_ATTR_TYPE_INTEGER;
+    GeisFilterableAttribute fa = &geis->region_filterable_attributes->store[i];
+    if (0 == strcmp(attr_name, fa->name))
+    {
+      return fa->type;
+    }
+  }
+  return GEIS_ATTR_TYPE_UNKNOWN;
+}
+
+
+
+GeisAttrType
+geis_get_special_attr_type(Geis geis, GeisString attr_name)
+{
+  GeisSize i;
+  for (i = 0; i < geis->special_filterable_attributes->count; ++i)
+  {
+    GeisFilterableAttribute fa = &geis->special_filterable_attributes->store[i];
+    if (0 == strcmp(attr_name, fa->name))
+    {
+      return fa->type;
+    }
   }
   return GEIS_ATTR_TYPE_UNKNOWN;
 }
