@@ -19,9 +19,11 @@
  * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  */
 #include "geis/geis.h"
+#include "geis_class.h"
 #include "geis_config.h"
 #include "geis_filter.h"
 #include "geis_logging.h"
+#include "geis_private.h"
 #include "geis_subscription.h"
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,16 +32,14 @@
 
 #define WINDOW_FILTER "geis v1"
 
-static Geis g_geis = NULL;
-static int  g_geis_refcount = 0;
-
 struct _GeisInstance
 {
+  Geis              geis;
   GeisSubscription  subscription;
   GeisFilter        window_filter;
-  GeisInputFuncs   *input_funcs;
+  GeisInputFuncs    input_funcs;
   GeisPointer       input_context;
-  GeisGestureFuncs *gesture_funcs;
+  GeisGestureFuncs  gesture_funcs;
   GeisPointer       gesture_cookie;
 };
 
@@ -77,6 +77,11 @@ struct _GeisV1AttrMap
   { GEIS_GESTURE_TYPE_TOUCH3,  GEIS_GESTURE_TOUCH,  3 },
   { GEIS_GESTURE_TYPE_TOUCH4,  GEIS_GESTURE_TOUCH,  4 },
   { GEIS_GESTURE_TYPE_TOUCH5,  GEIS_GESTURE_TOUCH,  5 },
+  { GEIS_GESTURE_TYPE_FLICK1,  GEIS_GESTURE_FLICK,  1 },
+  { GEIS_GESTURE_TYPE_FLICK2,  GEIS_GESTURE_FLICK,  2 },
+  { GEIS_GESTURE_TYPE_FLICK3,  GEIS_GESTURE_FLICK,  3 },
+  { GEIS_GESTURE_TYPE_FLICK4,  GEIS_GESTURE_FLICK,  4 },
+  { GEIS_GESTURE_TYPE_FLICK5,  GEIS_GESTURE_FLICK,  5 },
   { NULL,                      NULL,                0 }
 };
 
@@ -127,7 +132,8 @@ _generate_subscription_name(GeisXcbWinInfo* win_info)
  * Maps the geis v2 attrs in a frame to geis v1 attrs.
  */
 static GeisGestureAttr *
-_map_frame_to_attrs(GeisFrame     frame, 
+_map_frame_to_attrs(Geis          geis,
+                    GeisFrame     frame, 
                     GeisTouchSet  touchset,
                     GeisInteger  *class_id,
                     GeisSize     *attr_count)
@@ -137,6 +143,7 @@ _map_frame_to_attrs(GeisFrame     frame,
   GeisSize free_slot = 9; /* yes, it's magic! */
   GeisSize num_touches = geis_frame_touchid_count(frame) * 3; /* more magic! */
   GeisSize gga_size = free_slot + num_touches + num_attrs; 
+  GeisString gesture_name = NULL;
 
   *attr_count = 0;
 
@@ -148,6 +155,29 @@ _map_frame_to_attrs(GeisFrame     frame,
     goto final_exit;
   }
 
+  /* Determine the dominant class. */
+  {
+    GeisGestureClassBag classes = geis_gesture_classes(geis);
+    GeisSize classes_count = geis_gesture_class_bag_count(classes);
+    GeisSize i;
+    for (i = 0; i < classes_count; ++i)
+    {
+      GeisGestureClass gesture_class = NULL;
+      GeisInteger id = 0;
+
+      gesture_class = geis_gesture_class_bag_gesture_class(classes, i);
+      id = geis_gesture_class_id(gesture_class);
+      if (geis_frame_is_class(frame, gesture_class))
+      {
+	*class_id = id;
+	gesture_name = geis_gesture_class_name(gesture_class);
+	if (id > 100)
+	  break;
+      }
+    }
+  }
+
+  /* Map V2 attrs into V1 attrs. */
   for (i = 0; i < num_attrs; ++i)
   {
     GeisAttr attr = geis_frame_attr(frame, i);
@@ -155,7 +185,7 @@ _map_frame_to_attrs(GeisFrame     frame,
 
     if (0 == strcmp(attr_name, "geis gesture class id"))
     {
-      *class_id = geis_attr_value_to_integer(attr);
+      /* skip this, do not pass it on */
     }
     else if (0 == strcmp(attr_name, GEIS_GESTURE_ATTRIBUTE_DEVICE_ID))
     {
@@ -208,9 +238,10 @@ _map_frame_to_attrs(GeisFrame     frame,
     }
     else if (0 == strcmp(attr_name, GEIS_GESTURE_ATTRIBUTE_GESTURE_NAME))
     {
-      attrs[7].name        = attr_name;
-      attrs[7].type        = GEIS_ATTR_TYPE_STRING;
-      attrs[7].string_val = geis_attr_value_to_string(attr);
+      attrs[7].name       = attr_name;
+      attrs[7].type       = GEIS_ATTR_TYPE_STRING;
+      attrs[7].string_val = gesture_name ? gesture_name
+                                         : geis_attr_value_to_string(attr);
       ++*attr_count;
     }
     else if (0 == strcmp(attr_name, GEIS_GESTURE_ATTRIBUTE_TOUCHES))
@@ -275,7 +306,7 @@ final_exit:
 
 
 static void
-_v1_event_callback(Geis        geis __attribute((unused)),
+_v1_event_callback(Geis        geis,
                    GeisEvent   event,
                    GeisPointer context)
 {
@@ -324,13 +355,14 @@ _v1_event_callback(Geis        geis __attribute((unused)),
     {
       GeisSize attr_count = 0;
       GeisFrame frame = geis_group_frame(group, j);
-      if (!group)
+      if (!frame)
       {
 	geis_warning("can not extract frame %lu from group", j);
 	goto final_exit;
       }
 
-      GeisGestureAttr *attrs = _map_frame_to_attrs(frame,
+      GeisGestureAttr *attrs = _map_frame_to_attrs(geis,
+                                                   frame,
                                                    touchset,
                                                    &class_id,
                                                    &attr_count);
@@ -343,19 +375,19 @@ _v1_event_callback(Geis        geis __attribute((unused)),
       switch (geis_event_type(event))
       {
 	case GEIS_EVENT_GESTURE_BEGIN:
-	  v1_instance->gesture_funcs->start(v1_instance->gesture_cookie,
+	  v1_instance->gesture_funcs.start(v1_instance->gesture_cookie,
 	                                    class_id,
 	                                    geis_frame_id(frame),
 	                                    attr_count, attrs);
 	  break;
 	case GEIS_EVENT_GESTURE_UPDATE:
-	  v1_instance->gesture_funcs->update(v1_instance->gesture_cookie,
+	  v1_instance->gesture_funcs.update(v1_instance->gesture_cookie,
 	                                     class_id,
 	                                     geis_frame_id(frame),
 	                                     attr_count, attrs);
 	  break;
 	case GEIS_EVENT_GESTURE_END:
-	  v1_instance->gesture_funcs->finish(v1_instance->gesture_cookie,
+	  v1_instance->gesture_funcs.finish(v1_instance->gesture_cookie,
 	                                     class_id,
 	                                     geis_frame_id(frame),
 	                                     attr_count, attrs);
@@ -400,36 +432,32 @@ geis_init(GeisWinInfo* win_info, GeisInstance *geis_instance)
     window = (GeisInteger)(xcb_win_info->window_id);
   }
 
-  if (!g_geis)
+  if (win_info->win_type == geis_win_type_str(Test))
   {
-    if (win_info->win_type == geis_win_type_str(Test))
-    {
-      if (xcb_win_info)
-	g_geis = geis_new(GEIS_INIT_UTOUCH_MOCK_ENGINE,
-	                  GEIS_INIT_TRACK_GESTURE_CLASSES,
-	                  NULL);
-      else
-	g_geis = geis_new(GEIS_INIT_UTOUCH_MOCK_ENGINE, NULL);
-    }
+    if (xcb_win_info)
+      instance->geis = geis_new(GEIS_INIT_UTOUCH_MOCK_ENGINE,
+                                GEIS_INIT_TRACK_GESTURE_CLASSES,
+                                NULL);
     else
-    {
-      g_geis = geis_new(GEIS_INIT_UTOUCH_XCB,
-                        GEIS_INIT_TRACK_DEVICES,
-                        GEIS_INIT_TRACK_GESTURE_CLASSES,
-                        NULL);
-    }
-    if (!g_geis)
-    {
-      goto unwind_instance;
-    }
-    geis_register_event_callback(g_geis, _v1_event_callback, instance);
+      instance->geis = geis_new(GEIS_INIT_UTOUCH_MOCK_ENGINE, NULL);
   }
-  ++g_geis_refcount;
+  else
+  {
+    instance->geis = geis_new(GEIS_INIT_UTOUCH_XCB,
+                              GEIS_INIT_TRACK_DEVICES,
+                              GEIS_INIT_TRACK_GESTURE_CLASSES,
+                              NULL);
+  }
+  if (!instance->geis)
+  {
+    goto unwind_instance;
+  }
+  geis_register_event_callback(instance->geis, _v1_event_callback, instance);
 
-  instance->subscription = geis_subscription_new(g_geis,
+  instance->subscription = geis_subscription_new(instance->geis,
                           _generate_subscription_name(xcb_win_info),
                           GEIS_SUBSCRIPTION_CONT);
-  instance->window_filter = geis_filter_new(g_geis, WINDOW_FILTER);
+  instance->window_filter = geis_filter_new(instance->geis, WINDOW_FILTER);
   geis_filter_add_term(instance->window_filter,
                        GEIS_FILTER_REGION,
                        GEIS_REGION_ATTRIBUTE_WINDOWID, GEIS_FILTER_OP_EQ, window,
@@ -448,24 +476,19 @@ final_exit:
 
 
 GeisStatus
-geis_finish(GeisInstance geis_instance)
+geis_finish(GeisInstance instance)
 {
   GeisStatus status = GEIS_STATUS_SUCCESS;
-  geis_subscription_delete(geis_instance->subscription);
-  geis_filter_delete(geis_instance->window_filter);
-  free(geis_instance);
-  --g_geis_refcount;
-  if (!g_geis_refcount)
-  {
-    geis_delete(g_geis);
-  }
-  g_geis = NULL;
+  geis_subscription_delete(instance->subscription);
+  geis_filter_delete(instance->window_filter);
+  geis_delete(instance->geis);
+  free(instance);
   return status;
 }
 
 
 GeisStatus
-geis_configuration_supported(GeisInstance geis_instance __attribute__((unused)),
+geis_configuration_supported(GeisInstance geis_instance GEIS_UNUSED,
                              int          configuration_item)
 {
   GeisStatus status = GEIS_STATUS_NOT_SUPPORTED;
@@ -481,7 +504,7 @@ geis_configuration_supported(GeisInstance geis_instance __attribute__((unused)),
 
 
 GeisStatus
-geis_configuration_get_value(GeisInstance  geis_instance __attribute__((unused)),
+geis_configuration_get_value(GeisInstance  instance,
                              int           configuration_item, 
                              void         *value)
 {
@@ -495,7 +518,8 @@ geis_configuration_get_value(GeisInstance  geis_instance __attribute__((unused))
   switch (configuration_item)
   {
     case GEIS_CONFIG_UNIX_FD:
-      status = geis_get_configuration(g_geis, GEIS_CONFIGURATION_FD, value);
+      status = geis_get_configuration(instance->geis,
+                                      GEIS_CONFIGURATION_FD, value);
       break;
   }
   return status;
@@ -506,7 +530,7 @@ geis_configuration_get_value(GeisInstance  geis_instance __attribute__((unused))
  * Sets a feature configuration value.
  */
 GeisStatus
-geis_configuration_set_value(GeisInstance geis_instance __attribute__((unused)),
+geis_configuration_set_value(GeisInstance geis_instance GEIS_UNUSED,
                              int          configuration_item, 
                              void         *value)
 {
@@ -526,9 +550,9 @@ geis_configuration_set_value(GeisInstance geis_instance __attribute__((unused)),
 
 
 GeisStatus
-geis_event_dispatch(GeisInstance geis_instance __attribute__((unused)))
+geis_event_dispatch(GeisInstance instance)
 {
-  geis_dispatch_events(g_geis);
+  geis_dispatch_events(instance->geis);
   return GEIS_STATUS_SUCCESS;
 }
 
@@ -622,11 +646,11 @@ geis_subscribe(GeisInstance         instance,
 {
   GeisStatus result = GEIS_UNKNOWN_ERROR;
 
-  instance->gesture_funcs = funcs;
+  memcpy(&instance->gesture_funcs, funcs, sizeof(GeisGestureFuncs));
   instance->gesture_cookie = cookie;
 
   /* pump the geis event queue to force device and class events to be picked up */
-  while (GEIS_STATUS_CONTINUE == geis_dispatch_events(g_geis))
+  while (GEIS_STATUS_CONTINUE == geis_dispatch_events(instance->geis))
     ;
 
   if (input_list == GEIS_ALL_INPUT_DEVICES)
@@ -698,7 +722,9 @@ geis_unsubscribe(GeisInstance     instance,
 
 
 static void
-_v1_input_callback(Geis geis, GeisEvent event, void *context)
+_v1_input_callback(Geis       geis GEIS_UNUSED,
+                   GeisEvent  event,
+                   void      *context)
 {
   GeisInstance v1_instance = (GeisInstance)context;
   GeisDevice device;
@@ -759,12 +785,12 @@ _v1_input_callback(Geis geis, GeisEvent event, void *context)
   switch (geis_event_type(event))
   {
     case GEIS_EVENT_DEVICE_AVAILABLE:
-      v1_instance->input_funcs->added(v1_instance->input_context,
+      v1_instance->input_funcs.added(v1_instance->input_context,
                                       device_id,
                                       attrs);
       break;
     case GEIS_EVENT_DEVICE_UNAVAILABLE:
-      v1_instance->input_funcs->removed(v1_instance->input_context,
+      v1_instance->input_funcs.removed(v1_instance->input_context,
                                         device_id,
                                         attrs);
       break;
@@ -787,10 +813,10 @@ geis_input_devices(GeisInstance    instance,
 {
   GeisStatus result = GEIS_STATUS_SUCCESS;
 
-  instance->input_funcs = funcs;
+  memcpy(&instance->input_funcs, funcs, sizeof(GeisInputFuncs));
   instance->input_context = context;
 
-  geis_register_device_callback(g_geis, _v1_input_callback, instance);
+  geis_register_device_callback(instance->geis, _v1_input_callback, instance);
 
   return result;
 }
