@@ -34,6 +34,7 @@ const char* s_gestures[] = {
   GEIS_GESTURE_TYPE_ROTATE2, GEIS_GESTURE_TYPE_ROTATE3,
   GEIS_GESTURE_TYPE_TAP2, GEIS_GESTURE_TYPE_TAP3, GEIS_GESTURE_TYPE_TAP4,
   GEIS_GESTURE_TYPE_TOUCH3, GEIS_GESTURE_TYPE_TOUCH4,
+  GEIS_GESTURE_TYPE_FLICK2, GEIS_GESTURE_TYPE_FLICK3, GEIS_GESTURE_TYPE_FLICK4,
   NULL
 };
 
@@ -411,20 +412,33 @@ main(int argc, char* argv[])
 
   while(1)
   {
+    int i, fd;
+    int max_fd = 0;
     fd_set read_fds;
     FD_ZERO(&read_fds);
     FD_SET(0, &read_fds);
-    FD_SET(fd, &read_fds);
-    int sstat = select(fd+1, &read_fds, NULL, NULL, NULL);
+    for (i = 0; i < instance_table_size; ++i)
+    {
+      status = geis_configuration_get_value(instance_table[i],
+                                            GEIS_CONFIG_UNIX_FD, &fd);
+      max_fd = (fd > max_fd ? fd : max_fd);
+      FD_SET(fd, &read_fds);
+    }
+    int sstat = select(max_fd + 1, &read_fds, NULL, NULL, NULL);
     if (sstat < 0)
     {
       fprintf(stderr, "error %d in select(): %s\n", errno, strerror(errno));
       break;
     }
 
-    if (FD_ISSET(fd, &read_fds))
+    for (i = 0; i < instance_table_size; ++i)
     {
-      geis_event_dispatch(instance_table[0]);
+      status = geis_configuration_get_value(instance_table[i],
+                                            GEIS_CONFIG_UNIX_FD, &fd);
+      if (FD_ISSET(fd, &read_fds))
+      {
+	geis_event_dispatch(instance_table[i]);
+      }
     }
 
     if (FD_ISSET(0, &read_fds))
