@@ -8,6 +8,7 @@
 extern Suite *make_attr_suite();
 extern Suite *make_backend_event_posting_suite();
 extern Suite *make_backend_multiplexor_suite();
+extern Suite *make_backend_token_suite();
 extern Suite *make_device_suite();
 extern Suite *make_error_reporting_suite();
 extern Suite *make_event_queue_suite();
@@ -15,6 +16,7 @@ extern Suite *make_filter_suite();
 extern Suite *make_gesture_class_suite();
 extern Suite *make_region_suite();
 extern Suite *make_subscription_suite();
+extern Suite *make_timer_suite();
 
 
 int
@@ -29,12 +31,14 @@ main(int argc CK_ATTRIBUTE_UNUSED, char* argv[] CK_ATTRIBUTE_UNUSED)
   srunner_add_suite(sr, make_attr_suite());
   srunner_add_suite(sr, make_event_queue_suite());
   srunner_add_suite(sr, make_backend_multiplexor_suite());
+  srunner_add_suite(sr, make_backend_token_suite());
   srunner_add_suite(sr, make_backend_event_posting_suite());
   srunner_add_suite(sr, make_device_suite());
   srunner_add_suite(sr, make_gesture_class_suite());
   srunner_add_suite(sr, make_region_suite());
   srunner_add_suite(sr, make_filter_suite());
   srunner_add_suite(sr, make_subscription_suite());
+  srunner_add_suite(sr, make_timer_suite());
 
   srunner_set_log(sr, LOGFILE_PREFIX".log");
   srunner_set_xml(sr, LOGFILE_PREFIX".xml");
